@@ -1,43 +1,52 @@
 use strict;
 package inc;
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
-use Cwd();
-use Config();
+use 5.008001;
 
-my $corelists = {};
-my $plugins = {};
-my $inc = [@INC];
+use XXX;
+
+my $perl_init;
+my $perl_core;
+
+sub new {
+    my ($class, @spec) = @_;
+    my $init = run_perl_eval($perl_init);
+    my $self = bless {
+        spec => \@spec,
+        %$init,
+    }, $class;
+    return $self;
+}
 
 sub import {
     my ($class) = shift;
     return unless @_;
-    my $self = bless {}, $class;
-    @INC = $self->create_list(@_);
+    my $self = $class->new(@_);
+    @INC = $self->create_list;
     return;
 }
 
 sub list {
     my ($class) = shift;
-    die "'inc->list' requires at least one argument"
+    die "'inc->list()' requires at least one argument"
         unless @_;
-    my $self = bless {}, $class;
-    return $self->create_list(@_);
+    my $self = $class->new(@_);
+    return $self->create_list;
 }
 
 sub create_list {
     my ($self) = shift;
-    $self->{spec} = [@_];
     my $list = $self->{list} = [];
     $self->{inc} = [@INC];
-    while (my $next = $self->parse_spec(@_)) {
+    while (my $next = $self->parse_spec) {
         my ($name, @args) = @$next;
         if ($name =~ m!/!) {
             push @$list, $name;
         }
         else {
             my $method = "inc_$name";
-            die "No inc support found for '$name'"
+            die "No 'inc' support found for '$name'"
                 unless $self->can($method);
             push @$list, $self->$method(@args);
         }
@@ -86,52 +95,71 @@ sub lookup {
     return;
 }
 
-#------------------------------------------------------------------------------
-# Smart Objects
-#------------------------------------------------------------------------------
-sub inc_core {
-    my ($self, $version) = @_;
-    $version ||= $Config::Config{version};
-    my $hash = $corelists->{$version} ||= do {
-        +{
-            split /\s+/, `corelist -v $version //`
-        }
-    };
+sub run_perl_eval {
+    my ($perl, @argv) = @_;
+    local $ENV{PERL5OPT};
+
+    my $out = qx!$^X -e '$perl' @argv!;
+    return eval $out;
+}
+
+sub only_find {
+    my ($self, $hash) = @_;
     return sub {
         my ($self, $modpath) = @_;
         (my $modname = $modpath) =~ s!/!::!g;
         $modname =~ s!\.pm$!!;
         return unless $hash->{$modname};
-        return lookup($modpath, @$inc);
+        return lookup($modpath, @$INC);
     }
 }
 
+#------------------------------------------------------------------------------
+# Smart Objects
+#------------------------------------------------------------------------------
+sub inc_blib {
+    return 'blib/lib', 'blib/arch';
+}
+
+sub inc_cache {
+    my ($self) = @_;
+    die "inc 'cache' object not yet implemented";
+    return ();
+}
+
+sub inc_core {
+    my ($self, $version) = @_;
+    $version ||= $Config::Config{version};
+    my $hash = $self->{"corelists/$version"} ||=
+        run_perl_eval $perl_core, $version;
+    $self->only_find($hash);
+}
+
+sub inc_cwd {
+    my ($self) = @_;
+    return (
+        $self->{cwd},
+    );
+}
+
 sub inc_deps {
     my ($self, @module) = @_;
     die "inc 'deps' object not yet implemented";
 }
 
-sub inc_meta {
+sub inc_dot {
     my ($self) = @_;
-    die "inc 'meta' object not yet implemented";
+    return (
+        $self->{curdir},
+    );
 }
 
-sub inc_dist {
+my $hash_dzil;
+sub inc_dzil {
     my ($self) = @_;
-    die "inc 'dist' object not yet implemented";
-}
-
-sub inc_perl5lib {
-    return () unless defined $ENV{PERL5LIB};
-    return split /:/, $ENV{PERL5LIB};
-}
-
-sub inc_lib {
-    return Cwd::abs_path('lib');
-}
-
-sub inc_blib {
-    return 'blib/lib', 'blib/arch';
+    local $ENV{PERL5OPT};
+    $hash_dzil ||= +{ map { chomp; ($_, 1) } `dzil listdeps` };
+    $self->only_find($hash_dzil);
 }
 
 sub inc_inc {
@@ -140,34 +168,76 @@ sub inc_inc {
 }
 
 sub inc_INC {
-    return @$inc;
+    my ($self) = @_;
+    return @{$self->{INC}};
 }
 
-sub inc_priv {
-    die "inc 'priv' object not yet implemented";
+sub inc_lib {
+    return Cwd::abs_path('lib');
 }
 
-sub inc_not_priv {
-    die "inc '-priv' object not yet implemented";
-}
-
-sub inc_site {
-    die "inc 'site' object not yet implemented";
-}
-
-sub inc_not_site {
-    die "inc '-site' object not yet implemented";
-}
-
-sub inc_not {
-    die "inc 'not' object not yet implemented";
+sub inc_meta {
+    my ($self) = @_;
+    die "inc 'meta' object not yet implemented";
 }
 
 sub inc_none {
     return ();
 }
 
-sub inc_list {
+sub inc_not {
+    my ($self) = @_;
+    die "inc 'not' object requires one regex"
+        unless @_ == 1;
+    my $regex = qr/$_[0]/;
+    $self->{list} = [grep {ref or not($regex)} @{$self->{list}}];
+    return ();
+}
+
+sub inc_perl5lib {
+    return () unless defined $ENV{PERL5LIB};
+    return split /:/, $ENV{PERL5LIB};
+}
+
+sub inc_priv {
+    my ($self) = @_;
+    return (
+        $self->{archlib},
+        $self->{privlib},
+    );
+}
+
+sub inc_not_priv {
+    my ($self) = @_;
+    $self->{list} = [grep {
+        ref or not(
+            $_ eq $self->{archlib} or
+            $_ eq $self->{priv}
+        )
+    } @{$self->{list}}];
+    return ();
+}
+
+sub inc_site {
+    my ($self) = @_;
+    return (
+        $self->{sitearch},
+        $self->{sitelib},
+    );
+}
+
+sub inc_not_site {
+    my ($self) = @_;
+    $self->{list} = [grep {
+        ref or not(
+            $_ eq $self->{sitearch} or
+            $_ eq $self->{sitelib}
+        )
+    } @{$self->{list}}];
+    return ();
+}
+
+sub inc_show {
     my ($self) = @_;
     for (@{$self->{list}}) {
         print "$_\n";
@@ -175,4 +245,41 @@ sub inc_list {
     return ();
 }
 
+sub inc_zild {
+    my ($self) = @_;
+    die "inc 'zild' object not yet implemented";
+}
+
+#------------------------------------------------------------------------------
+# Perl scripts to run externally, so as not to load unintended modules into the
+# main process:
+#------------------------------------------------------------------------------
+$perl_init = <<'...';
+use Data::Dumper();
+use Cwd();
+use Config();
+use File::Spec;
+$Data::Dumper::Terse = 1;
+print Data::Dumper::Dumper +{
+    INC => \@INC,
+    archlib => $Config::Config{archlib},
+    privlib => $Config::Config{privlib},
+    sitearch => $Config::Config{sitearch},
+    sitelib => $Config::Config{sitelib},
+    curdir => File::Spec->curdir,
+    cwd => Cwd::cwd,
+};
+...
+
+$perl_core = <<'...';
+use Module::CoreList();
+use version();
+use Data::Dumper();
+
+my $version = shift @ARGV;
+$version = version->parse($version)->numify;
+$Data::Dumper::Terse = 1;
+print Data::Dumper::Dumper $Module::CoreList::version{$version};
+...
+
 1;
